@@ -13,6 +13,8 @@ macro_rules! impl_write_num {
                 let mut buf = [b'0'; $log10(<$u>::MAX) as usize + 1];
                 let mut n = *self;
                 let mut i = buf.len();
+                #[allow(clippy::indexing_slicing)]
+                // TODO(#1668) Clippy exceptions need docs or fixing.
                 while n != 0 {
                     i -= 1;
                     buf[i] = b'0' + (n % 10) as u8;
@@ -22,6 +24,8 @@ macro_rules! impl_write_num {
                     debug_assert_eq!(*self, 0);
                     i -= 1;
                 }
+                #[allow(clippy::indexing_slicing)]
+                // TODO(#1668) Clippy exceptions need docs or fixing.
                 let s = unsafe { core::str::from_utf8_unchecked(&buf[i..]) };
                 sink.write_str(s)
             }
@@ -142,11 +146,11 @@ impl Writeable for str {
     /// use writeable::Writeable;
     /// use std::borrow::Cow;
     ///
-    /// let cow = "foo".writeable_to_string();
+    /// let cow = "foo".write_to_string();
     /// assert!(matches!(cow, Cow::Borrowed(_)));
     /// ```
     #[inline]
-    fn writeable_to_string(&self) -> Cow<str> {
+    fn write_to_string(&self) -> Cow<str> {
         Cow::Borrowed(self)
     }
 }
@@ -163,7 +167,7 @@ impl Writeable for String {
     }
 
     #[inline]
-    fn writeable_to_string(&self) -> Cow<str> {
+    fn write_to_string(&self) -> Cow<str> {
         Cow::Borrowed(self)
     }
 }
@@ -185,8 +189,8 @@ impl<'a, T: Writeable + ?Sized> Writeable for &T {
     }
 
     #[inline]
-    fn writeable_to_string(&self) -> Cow<str> {
-        (*self).writeable_to_string()
+    fn write_to_string(&self) -> Cow<str> {
+        (*self).write_to_string()
     }
 }
 
